@@ -1,18 +1,19 @@
-# Modified: 23 May 2016 SDH
+# Modified: 03 Nov 2016
 
 cumBg <- function(
   # Main arguments
   dat,
   dat.type = 'vol',
-  comp = NULL,
+  comp = NULL, # Leave NULL for wide and both combos
   temp = NULL,
   pres = NULL,
   interval = TRUE,
+  data.struct = 'long', # long, wide, longcombo, widecombo
   # Column names for volumetric method
   id.name = 'id',
   time.name = 'time',
-  dat.name = dat.type,
-  comp.name = 'xCH4', 
+  dat.name = dat.type, # Will be used for first dat column for data.struct = 'wide'
+  comp.name = 'xCH4',  # Use for first comp col for data.struct = 'wide'
   # Additional arguments for manometric and gravimetric methods
   pres.resid = NULL,
   temp.init = NULL,
@@ -22,12 +23,14 @@ cumBg <- function(
   vol.hs.name = 'vol.hs',
   headcomp = 'N2',
   absolute = TRUE,
+  pres.amb = NULL,
   # Calculation method and other settings
   cmethod = 'removed',
   imethod = 'linear',
   extrap = FALSE,
   addt0 = TRUE,
   showt0 = TRUE,
+  ##gas = 'CH4',
   # Warnings and messages
   std.message = TRUE,
   check = TRUE,
@@ -35,14 +38,18 @@ cumBg <- function(
   temp.std = getOption('temp.std', 0.0), 
   pres.std = getOption('pres.std', 1.0),
   unit.temp = getOption('unit.temp', 'C'),
-  unit.pres = getOption('unit.pres', 'atm')
+  unit.pres = getOption('unit.pres', 'atm')##,
+  ##unit.vol = getOption('unit.vol', 'ml'),
+  ##unit.mass = getOption('unit.mass', 'g')
 ){
   # Check arguments
   checkArgClassValue(dat, 'data.frame')
   checkArgClassValue(dat.type, 'character', expected.values = c('vol', 'mass', 'pres', 'volume', 'pressure'))
   checkArgClassValue(comp, c('data.frame', 'integer', 'numeric', 'NULL'))
-  checkArgClassValue(temp, c('integer', 'numeric', 'NULL'))
-  checkArgClassValue(pres, c('integer', 'numeric', 'NULL'))
+  checkArgClassValue(temp, c('integer', 'numeric', 'character', 'NULL'))
+  checkArgClassValue(pres, c('integer', 'numeric', 'character', 'NULL'))
+  checkArgClassValue(interval, 'logical')
+  checkArgClassValue(data.struct, 'character', expected.values = c('long', 'wide', 'longcombo'))
   checkArgClassValue(id.name, 'character')
   checkArgClassValue(time.name, 'character')
   checkArgClassValue(dat.name, 'character')
@@ -66,23 +73,25 @@ cumBg <- function(
   checkArgClassValue(std.message, 'logical')
   checkArgClassValue(check, 'logical')
   checkArgClassValue(absolute, 'logical')
-  checkArgClassValue(interval, 'logical')
+  checkArgClassValue(pres.amb, c('integer', 'numeric', 'NULL'))
 
   # Hard-wire rh for now at least
   rh <- 1
 
   # Check column names in argument data frames
   # comp needs id (time) xCH4, time optional
-  if(!is.null(comp) && class(comp)=='data.frame') {
+  if(!is.null(comp) && class(comp)=='data.frame' && data.struct == 'long') {
     if(any(missing.col <- !c(id.name, comp.name) %in% names(comp))){
       stop('Specified column(s) in comp data frame (', deparse(substitute(comp)), ') not found: ', c(id.name, comp.name)[missing.col], '.')
     }
   }
 
   # dat (volume or mass)
-  if(any(missing.col <- !c(id.name, time.name, dat.name) %in% names(dat))){
-    stop('Specified columns in dat data frame (', deparse(substitute(dat)), ') not found: ', c(id.name, time.name, dat.name)[missing.col], '.')
-  } 
+  if(data.struct == 'long') {
+    if(any(missing.col <- !c(id.name, time.name, dat.name) %in% names(dat))){
+      stop('Specified columns in dat data frame (', deparse(substitute(dat)), ') not found: ', c(id.name, time.name, dat.name)[missing.col], '.')
+    } 
+  }
 
   # Check for headspace argument if it is needed
   if(is.null(headspace) & cmethod=='total') stop('cmethod is set to \"total\" but headspace argument is not provided.')
@@ -96,7 +105,11 @@ cumBg <- function(
   }
 
   # Check for input errors
-  # NTS: Add other checks here
+  #if(!is.null(comp) && class(comp)=='data.frame' && data.struct == 'long' && any(is.na(comp[, comp.name]))) stop('Missing data in comp data frame. See rows ', paste(which(is.na(comp[, comp.name])), collapse = ', '), '.')
+  # Drop missing comp rows
+
+  # NTS: Add other checks here (e.g., missing values elsewhere)
+
   if(check) {
     # Composition
     if(is.numeric(comp) | is.integer(comp)) {
@@ -113,67 +126,135 @@ cumBg <- function(
   # Convert date.type to lowercase so it is more flexible for users
   dat.type <- tolower(dat.type)
 
-  # Sort out composition data
-  mssg.no.time <- mssg.interp <- FALSE
-  # First sort so can find first observation for mass data to ignore it
-  dat <- dat[order(dat[, id.name], dat[, time.name]), ]
-  dat[, comp.name] <- NA
-  if(!is.null(comp) && class(comp)=='data.frame'){
-    # Interpolate gas composition to times of volume measurements
-    for(i in unique(dat[, id.name])) {
-      if(dat.type=='mass' & nrow(dat[dat[, id.name]==i, ])<2) stop('There are < 2 observations for reactor ', i,' but dat.type = \"mass\". 
-                                                                   You need at least 2 observations to apply the gravimetric method.')
-      dc <- comp[comp[, id.name]==i, ]
-      if(nrow(dc)==0) stop('No biogas composition data for reactor ', i,' so can\'t interpolate!') 
-      if(nrow(dc)>1) {
-	# If there is no time column
-	if(!time.name %in% names(comp)) stop('Problem with comp  (', deparse(substitute(comp)), 
-                                             '): a time column was not found but there is > 1 observation at least for reactor ',i, '.')
-	if(dat.type %in% c('vol', 'volume', 'pres', 'pressure')) {
-	  mssg.interp <- TRUE
-          dat[dat[, id.name]==i, comp.name] <- interp(dc[, time.name], dc[, comp.name], time.out = dat[dat[, id.name]==i, time.name], method = imethod, extrap = extrap)
-          # Set first value to zero if there is no biogas production (fixes problem with cmethod = total when there is a t0 observation included)
-          dat[dat[, id.name]==i & dat[, dat.name]==0, comp.name][1] <- 0
-	} else if (dat.type=='mass') {
-	  # Then ignore first point, since it isn't used anyway--this is just to avoid warning with interp if extrap = FALSE
-	  mssg.interp <- TRUE
-          dat[dat[, id.name]==i, comp.name][-1] <- interp(dc[, time.name], dc[, comp.name], time.out = dat[dat[, id.name]==i, time.name][-1], method = imethod, extrap = extrap)
-	}
-      } else { # If only one xCH4 value is available, use it for all dat obs if extrap = TRUE or times match, but warn if times don't match
-	if(!time.name %in% names(comp)) {
-	  # If there is no time column in comp
-	  mssg.no.time <- TRUE
-          dat[dat[, id.name]==i, comp.name] <- dc[, comp.name]
-	} else {
-	  # There is a time column in dc/comp
-          for(j in 1:nrow(dat[dat[, id.name]==i, ])) {
-	    if(j > 1 | dat.type=='vol') { # This just to avoid warning for first observation for mass data
-              if(dc[, time.name]==dat[dat[, id.name]==i, time.name][j]) { 
-	        # If times match
-                dat[dat[, id.name]==i, comp.name][j] <- dc[, comp.name]
-              } else {
-                if(extrap) {
-                  dat[dat[, id.name]==i, comp.name][j] <- dc[, comp.name]
-                } else {
-                  dat[dat[, id.name]==i, comp.name][j] <- NA
-      	          warning('Not enough xCH4 data (one observation) to interpolate for reactor ', i,' so results will be missing.\n If you prefer, you can use extrapolation by setting extrap = TRUE.')
-                }
-              }
-	    }
-          }
-	}
+  # Rearrange wide data
+  if(data.struct == 'wide') {
+
+    which.first.col <- which(names(dat) == dat.name)
+    dat.name <- dat.type
+
+    # Number of reactors
+    nr <- ncol(dat) - which.first.col + 1
+
+    # Reactor names taken from column names
+    ids <- names(dat)[which.first.col:nrow(dat)]
+
+    dat2 <- dat
+    dat <- dat[ , 1:which.first.col]
+    names(dat)[which.first.col] <- dat.name
+    dat <- data.frame(id = ids[1], dat)
+
+    for(i in 2:nr - 1) {
+      x <- dat2[ , c(1:(which.first.col - 1), which.first.col + i)]
+      names(x)[which.first.col] <- dat.name
+      x <- data.frame(id = ids[i + 1], x)
+      dat <- rbind(dat, x)
+    }
+
+    # Now for comp
+    if(!is.numeric(comp)) {
+      which.first.col <- which(names(comp) == comp.name)
+      comp.name <- 'xCH4'
+
+      # Number of reactors
+      if((ncol(comp) - which.first.col + 1) != nr) stop('Apparent number of reactors in dat and comp do not match. Problem with wide data.struct.')
+
+      comp2 <- comp
+      comp <- comp[ , 1:which.first.col]
+      names(comp)[which.first.col] <- comp.name
+      comp <- data.frame(id = ids[1], comp)
+
+      for(i in 2:nr - 1) {
+        x <- comp2[ , c(1:(which.first.col - 1), which.first.col + i)]
+        names(x)[which.first.col] <- comp.name
+        x <- data.frame(id = ids[i + 1], x)
+        comp <- rbind(comp, x)
       }
     }
-  } else if(!is.null(comp) && class(comp) %in% c('numeric', 'integer') && length(comp)==1) {
-    # Or if a single value is given, use it
-    message('Only a single value was provided for biogas composition (', comp, '), so applying it to all observations.')
-    dat[, comp.name] <- comp
-  } else {
-    # If no composition data is given, just use NA
-    dat[, comp.name] <- NA 
+
+    data.struct <- 'long'
+
   }
-  if(mssg.no.time) message('A time column was not found in comp (', deparse(substitute(comp)), '), and a single value was used for each reactor.')
-  if(mssg.interp) message('Biogas composition is interpolated.')
+
+  # If there are missing values in a longcombo data frame, switch to long
+  # NTS: this is not the most efficient approach, maybe revisit
+  if(data.struct == 'longcombo' && any(is.na(dat[, comp.name]))) {
+    comp <- dat[, c(id.name, time.name, comp.name)]
+    dat <- dat[, names(dat) != comp.name]
+
+    data.struct <- 'long'
+  }
+
+  # Sort out composition data if using long data.struct
+  # Skipped for longcombo (xCH4 already included in dat)
+  if(tolower(data.struct) == 'long') {
+
+    mssg.no.time <- mssg.interp <- FALSE
+    # First sort so can find first observation for mass data to ignore it
+    dat <- dat[order(dat[, id.name], dat[, time.name]), ]
+    dat[, comp.name] <- NA
+
+    if(!is.null(comp) && class(comp)=='data.frame'){
+
+      # Drop NAs from comp--this applies to wide, long, and longcombo data.struct
+      comp <- comp[!is.na(comp[, comp.name]), ]
+
+      # Interpolate gas composition to times of volume measurements
+      for(i in unique(dat[, id.name])) {
+        if(dat.type=='mass' & nrow(dat[dat[, id.name]==i, ])<2) stop('There are < 2 observations for reactor ', i,' but dat.type = \"mass\". 
+                                                                     You need at least 2 observations to apply the gravimetric method.')
+        dc <- comp[comp[, id.name]==i, ]
+        if(nrow(dc)==0) stop('No biogas composition data for reactor ', i,' so can\'t interpolate!') 
+        if(nrow(dc)>1) {
+          # If there is no time column
+          if(!time.name %in% names(comp)) stop('Problem with comp  (', deparse(substitute(comp)), 
+                                               '): a time column was not found but there is > 1 observation at least for reactor ',i, '.')
+          if(dat.type %in% c('vol', 'volume', 'pres', 'pressure')) {
+            mssg.interp <- TRUE
+            dat[dat[, id.name]==i, comp.name] <- interp(dc[, time.name], dc[, comp.name], time.out = dat[dat[, id.name]==i, time.name], method = imethod, extrap = extrap)
+            # Set first value to zero if there is no biogas production (fixes problem with cmethod = total when there is a t0 observation included)
+            dat[dat[, id.name]==i & dat[, dat.name]==0, comp.name][1] <- 0
+          } else if (dat.type=='mass') {
+            # Then ignore first point, since it isn't used anyway--this is just to avoid warning with interp if extrap = FALSE
+            mssg.interp <- TRUE
+            dat[dat[, id.name]==i, comp.name][-1] <- interp(dc[, time.name], dc[, comp.name], time.out = dat[dat[, id.name]==i, time.name][-1], method = imethod, extrap = extrap)
+          }
+        } else { # If only one xCH4 value is available, use it for all dat obs if extrap = TRUE or times match, but warn if times don't match
+          if(!time.name %in% names(comp)) {
+            # If there is no time column in comp
+            mssg.no.time <- TRUE
+            dat[dat[, id.name]==i, comp.name] <- dc[, comp.name]
+          } else {
+            # There is a time column in dc/comp
+            for(j in 1:nrow(dat[dat[, id.name]==i, ])) {
+              if(j > 1 | dat.type=='vol') { # This just to avoid warning for first observation for mass data
+                if(dc[, time.name]==dat[dat[, id.name]==i, time.name][j]) { 
+                  # If times match
+                  dat[dat[, id.name]==i, comp.name][j] <- dc[, comp.name]
+                } else {
+                  if(extrap) {
+                    dat[dat[, id.name]==i, comp.name][j] <- dc[, comp.name]
+                  } else {
+                    dat[dat[, id.name]==i, comp.name][j] <- NA
+        	          warning('Not enough ', comp.name, ' data (one observation) to interpolate for reactor ', i,' so results will be missing.\n If you prefer, you can use extrapolation by setting extrap = TRUE.')
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+    } else if(!is.null(comp) && class(comp) %in% c('numeric', 'integer') && length(comp)==1) {
+      # Or if a single value is given, use it
+      message('Only a single value was provided for biogas composition (', comp, '), so applying it to all observations.')
+      dat[, comp.name] <- comp
+    } else {
+      # If no composition data is given, just use NA
+      dat[, comp.name] <- NA 
+    }
+    if(mssg.no.time) message('A time column was not found in comp (', deparse(substitute(comp)), '), and a single value was used for each reactor.')
+    if(mssg.interp) message('Biogas composition is interpolated.')
+
+  } 
 
   # Add headspace if provided
   if(!is.null(headspace)) {
@@ -188,6 +269,23 @@ cumBg <- function(
     } else stop('headspace actual argument not recognized. What is it?')
   }
 
+  # Add temperature and pressure to dat if single numeric values were provided
+  if(!is.null(temp)) {
+    if(is.numeric(temp)) {
+      dat[, 'temperature'] <- temp
+      temp <- 'temperature'
+    } 
+  }
+
+  if(!is.null(pres)) {
+    if(is.numeric(pres)) {
+      dat[, 'pressure'] <- pres
+      pres <- 'pressure' 
+    } 
+  }
+
+
+
   # Volumetric
   # Function will work with vol and add columns
   if(dat.type %in% c('vol', 'volume', 'pres', 'pressure')) {
@@ -198,7 +296,7 @@ cumBg <- function(
     if(dat.type %in% c('vol', 'volume')) {
       message('Working with volume data, applying volumetric method.')
       if(!is.null(temp) & !is.null(pres)) {
-        dat$vBg <- stdVol(dat[, dat.name], temp = temp, pres = pres, rh = rh, pres.std = pres.std, 
+        dat$vBg <- stdVol(dat[, dat.name], temp = dat[, temp], pres = dat[, pres], rh = rh, pres.std = pres.std, 
                           temp.std = temp.std, unit.temp = unit.temp, unit.pres = unit.pres, 
                           std.message = std.message)
       } else {
@@ -218,42 +316,52 @@ cumBg <- function(
       }
 
       # If absolute != TRUE, calculate absolute pressure and add to dat
+      # WIP
       if(!absolute) {
-        dat[, aa <- paste0(dat.name, '.abs')] <- dat[, dat.name] + dat[, pres.resid]
+        if(is.null(pres.amb)) stop('Pressure measurements are GAUGE but pres.amb argument was not provided.')
+
+        dat[, aa <- paste0(dat.name, '.abs')] <- dat[, dat.name] + pres.amb
         dat.name <- aa
+
+        dat[, aa <- paste0(pres.resid, '.abs')] <- dat[, pres.resid] + pres.amb
+        pres.resid <- aa
       }
 
       # Add residual rh (after pressure measurement and venting)
-      dat$rh.resid <- dat[, pres.resid]/dat[, dat.name]
+      if(interval) {
+        dat$rh.resid <- dat[, pres.resid]/dat[, dat.name]
+      }
 
       # Sort to add *previous* residual pressure, rh, and temperature columns
       dat <- dat[order(dat[, id.name], dat[, time.name]), ]
 
-      # Add previous residual pressure, residual rh, and previous temperature columns
-      for(i in unique(dat[, id.name])) {
-        pr <- dat[dat[, id.name]==i, pres.resid]
-        rhr <- dat[dat[, id.name]==i, 'rh.resid']
-        dat[dat[, id.name]==i, 'pres.resid.prev'] <- c(pres.init, pr[-length(pr)])
-        dat[dat[, id.name]==i, 'rh.resid.prev'] <- c(rh.resid.init, rhr[-length(rhr)])
-        dat[dat[, id.name]==i, 'temp.prev'] <- c(temp.init, rep(temp, length(pr) - 1))
-      }
-
       # Finally, calculate volume of gas in bottle headspace
       if(interval) {
+
+        # Add previous residual pressure, residual rh, and previous temperature columns
+        for(i in unique(dat[, id.name])) {
+          pr <- dat[dat[, id.name]==i, pres.resid]
+          tt <- dat[dat[, id.name]==i, temp]
+          rhr <- dat[dat[, id.name]==i, 'rh.resid']
+          dat[dat[, id.name]==i, 'pres.resid.prev'] <- c(pres.init, pr[-length(pr)])
+          dat[dat[, id.name]==i, 'rh.resid.prev'] <- c(rh.resid.init, rhr[-length(rhr)])
+          dat[dat[, id.name]==i, 'temp.prev'] <- c(temp.init, tt[-length(tt)])
+        }
+
         # Second call (subtracted bit) is standardized volume in bottle headspace at end of previous measurement (after venting)
         # Result then may not be exactly volume vented in current measurement, but total new gas volume since last measurement (only differ if pres.resid differs)
-        dat$vBg <- stdVol(dat[, vol.hs.name], temp = temp, pres = dat[, dat.name], rh = rh, 
+        dat$vBg <- stdVol(dat[, vol.hs.name], temp = dat[, temp], pres = dat[, dat.name], rh = rh, 
                           pres.std = pres.std, temp.std = temp.std, unit.temp = unit.temp, 
                           unit.pres = unit.pres, std.message = FALSE, warn = FALSE) - 
-                   stdVol(dat[, vol.hs.name], temp = dat$temp.prev, pres = dat$pres.resid.prev, rh = dat$rh.resid.prev,  
+                   stdVol(dat[, vol.hs.name], temp = dat[, 'temp.prev'], pres = dat$pres.resid.prev, rh = dat$rh.resid.prev,  
                           pres.std = pres.std, temp.std = temp.std, unit.temp = unit.temp, 
                           unit.pres = unit.pres, std.message = std.message)
       } else {
         # Second call (subtracted bit) is original bottle headspace (standardized), assumed to start at first pres.resid (NTS: may have been sealed at RT--could be changed)
-        dat$vBg <- stdVol(dat[, vol.hs.name], temp = temp, pres = dat[, dat.name], rh = rh, 
+        dat$vBg <- stdVol(dat[, vol.hs.name], temp = dat[, temp], pres = dat[, dat.name], rh = rh, 
                           pres.std = pres.std, temp.std = temp.std, unit.temp = unit.temp, 
                           unit.pres = unit.pres, std.message = FALSE, warn = FALSE) - 
-                   stdVol(dat[, vol.hs.name], temp = temp, pres = pres.init, rh = rh.resid.init,  
+                   stdVol(dat[, vol.hs.name], temp = dat[, 'temp.init'], pres = pres.init, rh = rh.resid.init,  
                           pres.std = pres.std, temp.std = temp.std, unit.temp = unit.temp, 
                           unit.pres = unit.pres, std.message = std.message, warn = FALSE)
       }
@@ -261,26 +369,41 @@ cumBg <- function(
 
     # Calculate interval (or cum if interval = FALSE) gas production
     dat$vCH4 <- dat$vBg*dat[, comp.name]*vol.mol['CH4'] / (dat[, comp.name]*vol.mol['CH4'] +(1 - dat[, comp.name])*vol.mol['CO2'])  # CH4 and CO2 molar volumes in ml/mol
+    ##if(gas == 'CH4') {
+    ##  dat$vCH4 <- dat$vBg*dat[, comp.name]*vol.mol['CH4'] / (dat[, comp.name]*vol.mol['CH4'] +(1 - dat[, comp.name])*vol.mol['CO2'])  # CH4 and CO2 molar volumes in ml/mol
+    ##} else {
+    ##  dat$vCH4 <- dat$vBg*dat[, comp.name]*vol.mol['H2'] / (dat[, comp.name]*vol.mol['H2'] +(1 - dat[, comp.name])*vol.mol['CO2'])  # H2 and CO2 molar volumes in ml/mol
+    ##}
+
     ##if(addt0) dat[dat[, time.name]==0, 'vCH4'] <- 0 # Not clear why this was ever needed
 
-    # For cmethod = 'total', add headspace CH4 to vCH4
+    # For cmethod = 'total', calculate headspace CH4 to add for total below
     if(cmethod=='total') {
       if(dat.type %in% c('vol', 'volume')) {
-        warning('For cmethod = \"total\", headspace temperature is taken as temp (', temp, unit.temp, '), pressure as \"pres\" (', pres, unit.pres, '), and relative humidity as 1.0 (100%).')
-
-        # Small problem with rh assumption here. Will actually be < 1 after gas removal
+        # NTS: message needs to be fixed due to change in temp to column in dat
+        message('For cmethod = \"total\", headspace temperature is taken as temp (', temp, unit.temp, '), pressure as \"pres\" (', pres, unit.pres, '), and relative humidity as 1.0 (100%).')
+        # NTS: small problem with rh assumption here. Will actually be < 1 after gas removal
         # Also assume vol meas pressure pres = residual headspace pressure
         dat$vhsCH4 <- dat[, comp.name]*
-                      stdVol(dat[, vol.hs.name], temp = temp, pres = pres, rh = 1, pres.std = pres.std, 
+                      stdVol(dat[, vol.hs.name], temp = dat[, temp], pres = dat[, pres], rh = 1, pres.std = pres.std, 
                             temp.std = temp.std, unit.temp = unit.temp, unit.pres = unit.pres, 
                             std.message = std.message)
       }
 
       if(dat.type %in% c('pres', 'pressure')) {
-        dat$vhsCH4 <- dat[, comp.name]*
-                      stdVol(dat[, vol.hs.name], temp = temp, pres = dat[ , pres.resid], rh = dat$rh.resid,  
-                             pres.std = pres.std, temp.std = temp.std, unit.temp = unit.temp, 
-                             unit.pres = unit.pres, std.message = std.message)
+
+        if(interval) {
+          dat$vhsCH4 <- dat[, comp.name]*
+                        stdVol(dat[, vol.hs.name], temp = dat[, temp], pres = dat[ , pres.resid], rh = dat$rh.resid,  
+                               pres.std = pres.std, temp.std = temp.std, unit.temp = unit.temp, 
+                               unit.pres = unit.pres, std.message = std.message)
+        } else {
+          # For interval = FALSE, 
+          dat$vhsCH4 <- dat[, comp.name]*
+                        stdVol(dat[, vol.hs.name], temp = dat[, temp], pres = dat[ , pres.resid], rh = 1,
+                               pres.std = pres.std, temp.std = temp.std, unit.temp = unit.temp, 
+                               unit.pres = unit.pres, std.message = std.message)
+        }
       }
 
       # vhsCH4 is added to cvCH4 below
@@ -289,7 +412,7 @@ cumBg <- function(
 
     # Add t0 row if requested
     # Not added if there are already zeroes present!
-    if(addt0 & !class(dat[, time.name])[1] %in% c('numeric', 'integer')) addt0 <- FALSE
+    if(addt0 & !class(dat[, time.name])[1] %in% c('numeric', 'integer', 'difftime')) addt0 <- FALSE
     if(addt0 & !any(dat[, time.name]==0)) {
       t0 <- data.frame(id = unique(dat[, id.name]), time = 0)
       names(t0) <- c(id.name, time.name)
@@ -304,13 +427,13 @@ cumBg <- function(
 
     # Calculate delta t for rates
     dat <- dat[order(dat[, id.name], dat[, time.name]), ]
-    if(class(dat[, time.name])[1] %in% c('numeric', 'integer')) {
+    if(class(dat[, time.name])[1] %in% c('numeric', 'integer', 'difftime')) {
       dt <- c(NA, diff(dat[, time.name]))
     } else if(class(dat[, time.name])[1] %in% c('POSIXct', 'POSIXlt')) {
       dt <- c(NA, as.numeric(diff(dat[, time.name]), units = 'days'))
     } else {
       dt <- NA
-      warning('time column in dat data frame not recognized, so rates will not be calculated.')
+      warning('class of time column in dat data frame not recognized, so rates will not be calculated.')
     }
     # Set dt to NA for first observations for each reactor
     dt[c(TRUE, dat[, id.name][-1] != dat[, id.name][-nrow(dat)])] <- NA 
@@ -352,7 +475,6 @@ cumBg <- function(
       }
     }
 
-
     # Calculate rates for all cases 
     for(i in unique(dat[, id.name])) {
       dat[dat[, id.name]==i, 'rvBg'] <- dat[dat[, id.name]==i, 'vBg' ]/dt[dat[, id.name]==i]
@@ -366,12 +488,15 @@ cumBg <- function(
 
     # Sort and return results
     dat <- dat[order(dat[, id.name], dat[, time.name]), ]
-    if(is.null(comp)) {
+
+    if(is.null(comp) & data.struct != 'longcombo') { # NTS: revisit if data.struct is ever expanded
       dat <- dat[, ! names(dat) %in% c(comp.name, 'vCH4', 'cvCH4', 'rvCH4')]
     }
+
     if(all(is.na(dt))) {
       dat <- dat[, ! names(dat) %in% c('rvBg','rvCH4')]
     }
+
     rownames(dat) <- 1:nrow(dat)
 
     return(dat)
@@ -407,11 +532,11 @@ cumBg <- function(
       stop('Mass *gain* calculated for one or more observations. See ', paste('id.name column:', mass[whichones, id.name], ' and time.name column:', mass[whichones - 1, time.name], 'to', mass[whichones, time.name], sep = ' ', collapse = ', '), ' in dat data frame. ')
     }
 
-    mass[, c('vBg', 'vCH4')] <- mass2vol(mass = mass[, 'massloss'], xCH4 = mass[, comp.name], temp = temp, pres = pres, temp.std = temp.std, pres.std = pres.std, unit.temp = unit.temp, unit.pres = unit.pres, value = 'all', std.message = std.message)[, c('vBg', 'vCH4')]
+    mass[, c('vBg', 'vCH4')] <- mass2vol(mass = mass[, 'massloss'], xCH4 = mass[, comp.name], temp = mass[, temp], pres = mass[, pres], temp.std = temp.std, pres.std = pres.std, unit.temp = unit.temp, unit.pres = unit.pres, value = 'all', std.message = std.message)[, c('vBg', 'vCH4')]
     if(!is.null(headspace)) {
       # Apply initial headspace correction only for times 1 and 2 (i.e., one mass loss measurement per reactor)
       which1and2 <- sort(c(which(starts$start), which(starts$start) + 1) )
-      mass[which1and2, c('vBg', 'vCH4')] <- mass2vol(mass = mass$massloss[which1and2], xCH4 = mass[which1and2, comp.name], temp = temp, pres = pres, temp.std = temp.std, pres.std = pres.std, unit.temp = unit.temp, unit.pres = unit.pres, value = 'all', headspace = mass[which1and2, vol.hs.name], headcomp = 'N2', temp.init = temp.init, std.message = FALSE)[, c('vBg', 'vCH4')]
+      mass[which1and2, c('vBg', 'vCH4')] <- mass2vol(mass = mass$massloss[which1and2], xCH4 = mass[which1and2, comp.name], temp = mass[, temp], pres = mass[, pres], temp.std = temp.std, pres.std = pres.std, unit.temp = unit.temp, unit.pres = unit.pres, value = 'all', headspace = mass[which1and2, vol.hs.name], headcomp = 'N2', temp.init = temp.init, std.message = FALSE)[, c('vBg', 'vCH4')]
     }
     # Set time zero volumes to zero--necessary because xCH4 is always missing
     mass[mass$massloss==0, c('vBg', 'vCH4')] <- 0
